from entities.descriptors import DigitalVariableDescriptor, DigitalAlarmDescriptor
from firmwareconfig import FirmwareConfig as fwcfg


class DigitalVariableList:
    """A class to define the list of digital variables of the board"""

    # this class can be actually created only once
    _ready = False

    def __init__(self, _var_enum_index_start, _alarm_enum_index_start):
        assert self._ready is False, "Class {} should be created only once!".format(self.__class__.__name__)
        self._create(_var_enum_index_start, _alarm_enum_index_start)
        self._ready = True

    def count(self):
        return len(self.vars)

    def entries(self):
        return self.vars

    def get_var_enum_indexes(self):
        return [var.enum_index for var in self.vars]

    def get_alarm_enum_indexes(self):
        alarm_enum_indexes = []
        for var in self.vars:
            if var.alarm:
                alarm_enum_indexes.append(var.alarm.enum_index)
        return alarm_enum_indexes

    def _create(self, _var_enum_index_start, _alarm_enum_index_start):
        self.vars = list()

        alarm_enum_index = _alarm_enum_index_start
        var_enum_index = _var_enum_index_start

        self.vars.append(
            DigitalVariableDescriptor(
                firmware_config=fwcfg,
                name='FLAG_DUL_ALARMPOS1',
                ioname='DIN_ALRMPOS1',
                description='DUL current limiting on VEOC direct alarm 1',
                enum_index=var_enum_index,
                alarm=DigitalAlarmDescriptor(
                    enum_index=alarm_enum_index,
                    enable_default=None,    # requires a specific #define
                    timeout_default_ms=1.5
                )
            )
        )

        var_enum_index += 1
        if self.vars[-1].alarm:
            alarm_enum_index += 1

        self.vars.append(
            DigitalVariableDescriptor(
                firmware_config=fwcfg,
                name='FLAG_DUL_ALARMPOS2',
                ioname='DIN_ALRMPOS2',
                description='DUL current limiting on VEOC direct alarm 2',
                enum_index=var_enum_index,
                alarm=DigitalAlarmDescriptor(
                    enum_index=alarm_enum_index,
                    enable_default=None,    # requires a specific #define
                    timeout_default_ms=1.5
                )
            )
        )

        var_enum_index += 1
        if self.vars[-1].alarm:
            alarm_enum_index += 1

        self.vars.append(
            DigitalVariableDescriptor(
                firmware_config=fwcfg,
                name='FLAG_DUL_ALARMNEG1',
                ioname='DIN_ALRMNEG1',
                description='DUL current limiting on VEOC return alarm 1',
                enum_index=var_enum_index,
                alarm=DigitalAlarmDescriptor(
                    enum_index=alarm_enum_index,
                    enable_default=None,    # requires a specific #define
                    timeout_default_ms=1.5
                )
            )
        )

        var_enum_index += 1
        if self.vars[-1].alarm:
            alarm_enum_index += 1

        self.vars.append(
            DigitalVariableDescriptor(
                firmware_config=fwcfg,
                name='FLAG_DUL_ALARMNEG2',
                ioname='DIN_ALRMNEG2',
                description='DUL current limiting on VEOC return alarm 2',
                enum_index=var_enum_index,
                alarm=DigitalAlarmDescriptor(
                    enum_index=alarm_enum_index,
                    enable_default=None,    # requires a specific #define
                    timeout_default_ms=1.5
                )
            )
        )

        var_enum_index += 1
        if self.vars[-1].alarm:
            alarm_enum_index += 1

        self.vars.append(
            DigitalVariableDescriptor(
                firmware_config=fwcfg,
                name='FLAG_HYDRO_PWR_FAULT',
                ioname='DIN_HYDRO_PWR_FLT',
                description='Fault status acquisition of 12V ISO HYDRO power line',
                enum_index=var_enum_index,
                alarm=DigitalAlarmDescriptor(
                    enum_index=alarm_enum_index,
                    enable_default=True,
                    timeout_default_ms=1.5
                )
            )
        )

        var_enum_index += 1
        if self.vars[-1].alarm:
            alarm_enum_index += 1

        self.vars.append(
            DigitalVariableDescriptor(
                firmware_config=fwcfg,
                name='FLAG_LBL_PWR_FAULT',
                ioname='DIN_LBL_PWR_FLT',
                description='Fault status acquisition of 12V ISO LBL power line',
                enum_index=var_enum_index,
                alarm=DigitalAlarmDescriptor(
                    enum_index=alarm_enum_index,
                    enable_default=True,
                    timeout_default_ms=1.5
                )
            )
        )

        var_enum_index += 1
        if self.vars[-1].alarm:
            alarm_enum_index += 1

        self.vars.append(
            DigitalVariableDescriptor(
                firmware_config=fwcfg,
                name='FLAG_GLRA_FAULT',
                ioname='DIN_GLR_A_FLT',
                description='Fault status acquisition of 12V GLENAIR-A power line',
                enum_index=var_enum_index,
                alarm=None
                # alarm=DigitalAlarmDescriptor(
                #     enum_index=alarm_enum_index,
                #     enable_default=False,
                #     timeout_default_ms=1.5
                # )
            )
        )

        var_enum_index += 1
        if self.vars[-1].alarm:
            alarm_enum_index += 1

        self.vars.append(
            DigitalVariableDescriptor(
                firmware_config=fwcfg,
                name='FLAG_GLRB_FAULT',
                ioname='DIN_GLR_B_FLT',
                description='Fault status acquisition of 12V GLENAIR-B power line',
                enum_index=var_enum_index,
                alarm=None
                # alarm=DigitalAlarmDescriptor(
                #     enum_index=alarm_enum_index,
                #     enable_default=False,
                #     timeout_default_ms=1.5
                # )
            )
        )

        var_enum_index += 1
        if self.vars[-1].alarm:
            alarm_enum_index += 1

        self.vars.append(
            DigitalVariableDescriptor(
                firmware_config=fwcfg,
                name='FLAG_POWERBOARD_FAULT',
                ioname='DIN_PWB_FLT',
                description='Fault status acquisition of 12V POWER BOARD power line',
                enum_index=var_enum_index,
                alarm=None
                # alarm=DigitalAlarmDescriptor(
                #     enum_index=alarm_enum_index,
                #     enable_default=False,
                #     timeout_default_ms=1.5
                # )
            )
        )

        var_enum_index += 1
        if self.vars[-1].alarm:
            alarm_enum_index += 1

        self.vars.append(
            DigitalVariableDescriptor(
                firmware_config=fwcfg,
                name='FLAG_GLRA_GOOD',
                ioname='DIN_GLR_A_PG',
                description='Power good status acquisition of 12V GLENAIR-A power line',
                enum_index=var_enum_index,
                alarm=None
                # alarm=DigitalAlarmDescriptor(
                #     enum_index=alarm_enum_index,
                #     enable_default=False,
                #     timeout_default_ms=1.5
                # )
            )
        )

        var_enum_index += 1
        if self.vars[-1].alarm:
            alarm_enum_index += 1

        self.vars.append(
            DigitalVariableDescriptor(
                firmware_config=fwcfg,
                name='FLAG_GLRB_GOOD',
                ioname='DIN_GLR_B_PG',
                description='Power good status acquisition of 12V GLENAIR-B power line',
                enum_index=var_enum_index,
                alarm=None
                # alarm=DigitalAlarmDescriptor(
                #     enum_index=alarm_enum_index,
                #     enable_default=False,
                #     timeout_default_ms=1.5
                # )
            )
        )

        var_enum_index += 1
        if self.vars[-1].alarm:
            alarm_enum_index += 1

        self.vars.append(
            DigitalVariableDescriptor(
                firmware_config=fwcfg,
                name='FLAG_POWERBOARD_GOOD',
                ioname='DIN_PWB_PG',
                description='Power good status acquisition of 12V POWER BOARD power line',
                enum_index=var_enum_index,
                alarm=None
                # alarm=DigitalAlarmDescriptor(
                #     enum_index=alarm_enum_index,
                #     enable_default=False,
                #     timeout_default_ms=1.5
                # )
            )
        )



if __name__ == '__main__':
    digital_variables = DigitalVariableList()
    print digital_variables.vars[0].name




