from switches import SwitchList
from commands import CommandList # need to be called to initialize all enums
from analogvariables import AnalogVariableList
from digitalvariables import DigitalVariableList
from userpins import UserPinList
from entities.descriptors import PayloadFieldEnum


def generate_switches_header(_switches):

    includes = """
#include "../../errors.h"
"""

    ####################################################################################################################
    typedef_tpl = """
typedef struct {
[typedef_entries]
} switches_t;
"""

    typedef_entry_tpl = """    enum_switchstate_t state_[name]; // status of switch [name], [desc]"""

    typedef_entries = list()
    for switch in _switches.entries():
        typedef_entry = typedef_entry_tpl
        typedef_entry = typedef_entry.replace('[name]', switch.name)
        typedef_entry = typedef_entry.replace('[desc]', switch.description)
        typedef_entries.append(typedef_entry)
    typedef = typedef_tpl
    typedef = typedef.replace('[typedef_entries]', '\n'.join(typedef_entries))

    ####################################################################################################################
    extern = """extern switches_t switches;"""

    ####################################################################################################################
    functionprotos_tpl = """
// control functions for switch [name], [desc]
void switch_[name]_set_closed(void);
void switch_[name]_set_open(void);
void switch_[name]_set(enum_switchstate_t _state);
enum_switchstate_t switch_[name]_get(void); 
"""

    functionprotos_entries = list()
    for switch in _switches.entries():
        functionprotos_entry = functionprotos_tpl
        functionprotos_entry = functionprotos_entry.replace('[name]', switch.name)
        functionprotos_entry = functionprotos_entry.replace('[desc]', switch.description)
        functionprotos_entries.append(functionprotos_entry)
    functionprotos = '\n'.join(functionprotos_entries)

    ####################################################################################################################
    functions_by_enum_tpl = """
// return the switch state set function by switch enum
typedef void (*switch_state_set_func_t)(enum_switchstate_t _state);
switch_state_set_func_t switch_get_function_set_by_enum(enum_switchnum_t _switch_index);

// return the switch state get function by switch enum
typedef enum_switchstate_t (*switch_state_get_func_t)(void);
switch_state_get_func_t switch_get_function_get_by_enum(enum_switchnum_t _switch_index);
"""

    functions_by_enum = functions_by_enum_tpl

    return '\n'.join([includes, typedef, extern, functionprotos, functions_by_enum])


def generate_switches_source(_switches, _switch_enum):

    assert isinstance(_switch_enum, PayloadFieldEnum)

    includes = """"""

    ####################################################################################################################
    instance_tpl = """
switches_t switches = {
[instance_entries]
};
"""

    instance_entry_tpl = """    .state_[name] = SWITCHSTATE_OPEN"""

    instance_entries = list()
    for switch in _switches.entries():
        instance_entry = instance_entry_tpl
        instance_entry = instance_entry.replace('[name]', switch.name)
        instance_entries.append(instance_entry)
    instance = instance_tpl
    instance = instance.replace('[instance_entries]', ',\n'.join(instance_entries))

    ####################################################################################################################
    functionimpls_tpl = """
// close switch [name] and update switch structure
void switch_[name]_set_closed(void) {
    [ioname]_SetHigh();
    switches.state_[name] = SWITCHSTATE_CLOSED;
}

// open switch [name] and update switch structure
void switch_[name]_set_open(void) {
    [ioname]_SetLow();
    switches.state_[name] = SWITCHSTATE_OPEN;
}

// set state of switch [name] and update switch structure
void switch_[name]_set(enum_switchstate_t _state) {
    if(_state == SWITCHSTATE_CLOSED) {
        switch_[name]_set_closed();
    }else{
        switch_[name]_set_open();
    }
}

// get switch [name] current status
enum_switchstate_t switch_[name]_get(void) {
    return switches.state_[name];
} 
"""

    functionimpls_entries = list()
    for switch in _switches.entries():
        functionimpls_entry = functionimpls_tpl
        functionimpls_entry = functionimpls_entry.replace('[name]', switch.name)
        functionimpls_entry = functionimpls_entry.replace('[ioname]', switch.ioname)
        functionimpls_entries.append(functionimpls_entry)
    functionimpls = '\n'.join(functionimpls_entries)

    ####################################################################################################################
    functions_by_enum_tpl = """
// return the switch state set function by switch enum
switch_state_set_func_t switch_get_function_set_by_enum(enum_switchnum_t _switch_index) {
[bodyset]
}

// return the switch state get function by switch enum
switch_state_get_func_t switch_get_function_get_by_enum(enum_switchnum_t _switch_index) {
[bodyget]
}
"""

    bodyset = """    switch(_switch_index) {\n"""
    bodyget = """    switch(_switch_index) {\n"""
    enumobj = _switch_enum
    for idx in range(len(enumobj.value_list)):
        # print enumobj.value_list[idx]
        bodyset += '        case {}_{}: return switch_{}_set;\n'.format(enumobj.name, enumobj.value_list[idx], _switches.entries()[idx].name)
        bodyget += '        case {}_{}: return switch_{}_get;\n'.format(enumobj.name, enumobj.value_list[idx], _switches.entries()[idx].name)
    bodyset += '    }\n'
    bodyset += '    // this should never be reached\n'
    bodyset += '    error_die_forever();\n'
    bodyset += '    // but, just in case (if we neutralize the error function):\n'
    bodyset += '    return switch_{}_set;'.format(_switches.entries()[-1].name)
    bodyget += '    }\n'
    bodyget += '    // this should never be reached\n'
    bodyget += '    error_die_forever();\n'
    bodyget += '    // but, just in case (if we neutralize the error function):\n'
    bodyget += '    return switch_{}_get;'.format(_switches.entries()[-1].name)

    functions_by_enum = functions_by_enum_tpl
    functions_by_enum = functions_by_enum.replace('[bodyset]', bodyset)
    functions_by_enum = functions_by_enum.replace('[bodyget]', bodyget)

    return '\n'.join([includes, instance, functionimpls, functions_by_enum])


if __name__ == '__main__':
    switches = SwitchList()
    analogvariables = AnalogVariableList()
    digitalvariables = DigitalVariableList()
    userpins = UserPinList()
    commands = CommandList(switches, analogvariables, digitalvariables, userpins)
    switch_enum = PayloadFieldEnum.get_persistent_list()['SWITCHNUM']

    print '\n' * 4
    print '=' * 64
    print generate_switches_header(switches)
    print generate_switches_source(switches, switch_enum)
